'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var ramda = require('ramda');
var ramdaExtension = require('ramda-extension');
var namespaces = require('@redux-tools/namespaces');
var invariant = _interopDefault(require('invariant'));
var utils = require('@redux-tools/utils');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var createRawEntries = function createRawEntries(injectables) {
  // NOTE: `isFunction` from ramda-extension returns `false` for `jest.fn()`.
  if (typeof injectables === 'function') {
    return [{
      path: [],
      value: injectables
    }];
  }

  if (ramdaExtension.isArray(injectables)) {
    return ramda.chain(createRawEntries, injectables);
  }

  if (ramdaExtension.isObject(injectables)) {
    var createRawEntriesFromPair = function createRawEntriesFromPair(_ref) {
      var key = _ref[0],
          injectable = _ref[1];
      var entries = createRawEntries(injectable);

      var prependEntryPath = function prependEntryPath(entry) {
        return _extends({}, entry, {
          path: [key].concat(entry.path)
        });
      };

      return ramda.map(prependEntryPath, entries);
    };

    return ramda.o(ramda.chain(createRawEntriesFromPair), ramda.toPairs)(injectables);
  }

  return [];
};
/**
 * Converts the input of `store.injectSomething()` or `store.ejectSomething()`
 * to an array of standalone entries.
 *
 * @param {*} injectables injectables to create the entries from
 * @param {Object} props props to store in all entries, i.e. `namespace` and `feature`
 * @returns {Object[]} an array of entries
 */


var createEntries = function createEntries(injectables, _temp) {
  var _ref2 = _temp === void 0 ? {} : _temp,
      feature = _ref2.feature,
      namespace = _ref2.namespace;

  var sanitizedProps = ramdaExtension.rejectNil({
    namespace: namespace,
    feature: feature != null ? feature : namespace ? namespaces.DEFAULT_FEATURE : null
  });
  var rawEntries = createRawEntries(injectables);
  var addPropsToEntries = ramda.map(ramda.mergeRight(sanitizedProps));
  return addPropsToEntries(rawEntries);
};

var enhanceStore = function enhanceStore(prevStore, storeInterface, _temp) {
  var _extends2;

  var _ref = _temp === void 0 ? {} : _temp,
      _ref$onEjected = _ref.onEjected,
      onEjected = _ref$onEjected === void 0 ? ramdaExtension.noop : _ref$onEjected,
      _ref$onInjected = _ref.onInjected,
      onInjected = _ref$onInjected === void 0 ? ramdaExtension.noop : _ref$onInjected;

  invariant(ramdaExtension.isObject(prevStore), 'You must pass a Redux store as the first argument to `enhanceStore()`');
  invariant(ramdaExtension.isObject(storeInterface), 'You must pass a store interface as the second argument to `enhanceStore()`');
  var injectionKey = storeInterface.injectionKey,
      ejectionKey = storeInterface.ejectionKey,
      getEntries = storeInterface.getEntries,
      setEntries = storeInterface.setEntries,
      type = storeInterface.type;
  var _prevStore$dispatch = prevStore.dispatch,
      dispatch = _prevStore$dispatch === void 0 ? ramdaExtension.noop : _prevStore$dispatch;
  var actionType = ramdaExtension.toScreamingSnakeCase(type);

  var inject = function inject(injectables, props) {
    if (props === void 0) {
      props = {};
    }

    var entries = createEntries(injectables, props);
    var nextEntries = ramda.concat(getEntries(nextStore), entries);
    setEntries(nextEntries, nextStore);
    onInjected({
      injectables: injectables,
      props: props,
      entries: entries
    });
    dispatch({
      type: "@redux-tools/" + actionType + "_INJECTED",
      payload: ramda.pluck('path', entries),
      meta: props
    });
  };

  var eject = function eject(injectables, props) {
    if (props === void 0) {
      props = {};
    }

    var entries = createEntries(injectables, props);
    var nextEntries = utils.withoutOnce(entries, getEntries(nextStore));
    setEntries(nextEntries, nextStore);
    onEjected({
      injectables: injectables,
      props: props,
      entries: entries
    });
    dispatch({
      type: "@redux-tools/" + actionType + "_EJECTED",
      payload: ramda.pluck('path', entries),
      meta: props
    });
  };

  var nextStore = _extends({}, prevStore, (_extends2 = {}, _extends2[injectionKey] = inject, _extends2[ejectionKey] = eject, _extends2));

  setEntries([], nextStore);
  return nextStore;
};

var makeStoreInterface = function makeStoreInterface(type) {
  invariant(type, 'The type of the injectables must be defined.');
  return {
    type: type,
    injectionKey: "inject" + ramdaExtension.toPascalCase(type),
    ejectionKey: "eject" + ramdaExtension.toPascalCase(type),
    getEntries: ramda.o(ramdaExtension.defaultToEmptyArray, ramda.path(['entries', type])),
    setEntries: ramda.curry(function (entries, store) {
      var _extends2;

      return store.entries = _extends({}, store.entries, (_extends2 = {}, _extends2[type] = entries, _extends2));
    })
  };
};

exports.createEntries = createEntries;
exports.enhanceStore = enhanceStore;
exports.makeStoreInterface = makeStoreInterface;
